local MAJOR, MINOR = "LibGuildAlts-1.0", tonumber("r7") or 9999;
local lib, oldminor = LibStub:NewLibrary(MAJOR, MINOR);
if not lib then 
	return; -- No Upgrade needed. 
end 

local _G = getfenv(0)

local Alts = {}
local Mains = nil -- reverse lookup table
local tinsert = _G.tinsert
local unpack = _G.unpack

local libalts = GetLibrary("LibAlts-1.0");

local function generateRevLookups()
	Mains = {}
	for k,v in pairs(Alts) do
		for i,a in ipairs(v) do
			Mains[a] = k
		end
	end
end
 
local function setAlt(main, alt)
	if (not main) or (not alt) then return end
	Mains = nil

	main = main:lower()
	alt = alt:lower()

	Alts[main] = Alts[main] or {}
	tinsert(Alts[main], alt)
		
	libalts:SetAlt(main, alt);
end

--- API for addons retrieve alts
-- @name :GetAlt 
-- @usage Get a list of alts for a given character 
-- @param main Name of the main character
-- @return  list list of alts 
function lib:GetAlts(main)
	if not main then return end

	main = main:lower()

	if not Alts[main] or #Alts[main] == 0 then
		return nil
	end

	return unpack(Alts[main])
end

--- API for addons to retrieve mains
-- @name :IsMain 
-- @usage Get main for a given alt character 
-- @param alt Name of the alt character
-- @return string the main character 
function lib:GetMain(alt)
	if not alt then return end

	alt = alt:lower()
	if not Mains then
		generateRevLookups()
	end

	return Mains[alt]
end

--- API for addons to test Main/Alt status
-- @name :IsMain
-- @usage Test if a character is a main
-- @param main Name of the character
-- @return boolean is this a main character
function lib:IsMain(main)
	if not main then return end
	return Alts[main:lower()] and true or false
end


--- API for addons to test Main/Alt status
-- @name :IsAlt
-- @usage Test if a character is a alt
-- @param alt Name of the character
-- @return boolean is this a alt character
function lib:IsAlt(alt)
	if not alt then return end
	
    if not Mains then
		generateRevLookups()
	end

	return Mains[alt:lower()] and true or false
end

if not lib.loaded then 
	local i;
	local numGuildMembers = GetNumGuildMembers(true);
	for i = 1, numGuildMembers do
		local pattern = "Main:%s*[^%s]+";
		local name, _, _, _, _, _, note, officernote = GetGuildRosterInfo(i);
		local _, _, main = officernote:find(pattern);
		if not main then
			_, _, main = note:find(pattern);
		end
		if main then 
			setAlt(main, name);
		end
	end
	lib.loaded = true;
end